<?php
/**
 * Manage all of the gift certificates on record
 */
function uc_gift_certificate_manage($view_type = 'manage') {
  $query = 'SELECT gc.*, u.name AS purchasername, u2.name AS username, u.mail AS purchasermail, u2.mail AS usermail
            FROM {uc_gift_certificates} AS gc
            LEFT JOIN {users} AS u on gc.purchaser_id = u.uid
            LEFT JOIN {users} AS u2 on gc.user_id = u2.uid';

  // $result = db_query($query);
  $rows = array();

  $header = array(
    array('data' => t('Name'), 'field' => 'gc.name'),
    array('data' => t('Gift Certificate Purchaser'), 'field' => 'purchasername', 'width' => '230'),
    array('data' => t('Gift Certificate Recipient'), 'width' => '230', 'field' => 'username'),
    array('data' => t('Value'), 'width' => '60', 'field' => 'gc.value'),
    array('data' => t('Action'), 'width' => 50),
    array('sort' => 'asc'),
  );

  $result = pager_query($query.tablesort_sql($header), 200, 0, NULL);

  while ($row = db_fetch_object($result)) {
    $rows[] = array($row->name,
                    $row->purchasername." (".$row->purchasermail.")",
                    $row->username." (".$row->usermail.")",
                    $row->value,
                    l(t('edit'), "admin/store/gift_certificates/$row->certificate_id/edit") . ' ' . l(t('delete'),"admin/store/gift_certificates/$row->certificate_id/delete"),
                  );
  }

  $output = theme('table', $header, $rows).theme('pager', NULL, 20, 0);

  // No gift certificates
  if ($output == null) {
    $output = "<p>There are currently no gift certificates in the system.</p>";
  }

  return $output;
}

/**
 * Implementation of hook_form
 *
 * Show the form to add a new certificate
 */
function uc_gift_certificate_add_form($form_state, $certificate_id) {
  $is_add = $certificate_id == 'add';
  $msg_to_recipient = NULL;

  if (!$is_add) {
    $cert = uc_gift_certificate_load($certificate_id);

    $form['certificate_id'] = array(
      '#type' => 'value',
      '#value' => $cert->certificate_id,
    );
    if (!empty($cert->order_id)) {
      $order = uc_order_load($cert->order_id);
      $form['order_id'] = array(
        '#type' => 'item',
        '#value' => t('This certificate was created as part of order !order_link', array('!order_link' => l('#' . $cert->order_id, 'admin/store/orders/' . $cert->order_id))),
      );
      $msg_attrib = uc_attribute_load(variable_get('uc_gift_certificate_message_aid', ''));
      foreach ($order->products as $product) {
        if ($product->order_product_id == $cert->order_product_id && !empty($product->data['attributes'][$msg_attrib->name][0])) {
          $msg_to_recipient = $product->data['attributes'][$msg_attrib->name][0];
          break;
        }
      }
    }
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Gift Certificate Name'),
    '#default_value' => $cert->name,
    '#required' => true,
  );
  $form['cert_code'] = array(
    '#prefix' => t('<b>Certificate Code:</b><br/>'),
    '#value' => $cert->cert_code,
  );

  // If not using autocomplete
  if(!variable_get('uc_use_autocomplete_user_fields', true)) {
    $form['purchaser_id'] = array(
      '#type' => 'select',
      '#title' => t('Purchaser'),
      '#default_value' => $cert->purchaser_id,
      '#options' => uc_gift_certificate_load_users(),
      '#description' => t('Username of the purchaser of the gift certificate'),
    );

    $form['user_id'] = array(
      '#type' => 'select',
      '#title' => t('Recipient'),
      '#default_value' => $cert->user_id,
      '#options' => uc_gift_certificate_load_users(),
      '#description' => t('Username of the recipient of the gift certificate'),
    );
  }
  else {
    $form['purchaser_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Purchaser'),
      '#default_value' => uc_gift_certificate_username_from_uid($cert->purchaser_id),
      '#description' => t('Username of the purchaser of gift certificate'),
      '#autocomplete_path' => 'user/autocomplete',
    );

    $form['user_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Recipient'),
      '#default_value' => uc_gift_certificate_username_from_uid($cert->user_id),
      '#autocomplete_path' => 'user/autocomplete',
      '#description' => t('Username of the recipient of the gift certificate'),
    );
  }

  if ($is_add) {
    $form['new_user'] = array(
      '#type' => 'fieldset',
      '#title' => t('Create a new user'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['new_user']['new_user_checkbox'] = array(
      '#type' => 'checkbox',
      '#title' => t('Create a new user account for the recipient'),
      '#description' => t('Please check here to create a new user account for the recipient. You must then enter the email of the new user below.'),
    );
    $form['new_user']['new_user_email'] = array(
      '#type' => 'textfield',
      '#size' => 25,
      '#title' => t('New user\'s email address'),
      '#description' => t(''),
    );
  }

  $form['value'] = array(
    '#type' => 'textfield',
    '#title' => t('Value'),
    '#default_value' => $cert->value,
    '#size' => 25,
    '#description' => t('Amount this gift certificate is worth.'),
    '#required' => true,
  );

  $form['msg_to_recipient'] = array(
    '#type' => 'textarea',
    '#title' => t('Message to recipient'),
    '#default_value' => $msg_to_recipient,
  );
  $form['mail_user_checkbox'] = array(
    '#type' => 'checkbox',
    '#title' => t('Email the recipient'),
    '#description' => t('Check here to email the recipient the gift certificate.'),
  );
  $form['op'] = array('#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Implementation of hook_form_validate()
 */
function uc_gift_certificate_add_form_validate($form, &$form_state) {
  $use_auto = variable_get('uc_use_autocomplete_user_fields', true);
  $is_new_user = $form_state['values']['new_user_checkbox'];
  if ($use_auto) {
    $form_state['values']['purchaser_id'] = uc_gift_certificate_uid_from_username($form_state['values']['purchaser_id']);
    $form_state['values']['user_id'] =  uc_gift_certificate_uid_from_username($form_state['values']['user_id']);
  }

  if (!db_result(db_query('SELECT uid FROM {users} WHERE uid != 0 AND uid = %d', $form_state['values']['purchaser_id']))) {
    form_set_error('purchaser_id', t('Purchaser is not a valid user.'));
  }
  if (!$is_new_user && !db_result(db_query('SELECT uid FROM {users} WHERE uid != 0 AND uid = %d', $form_state['values']['user_id']))) {
    form_set_error('user_id', t('Recipient is not a valid user.'));
  }
  if (empty($form_state['values']['value']) || !is_numeric($form_state['values']['value'])) {
    form_set_error('value', t('Value is not a valid amount.'));
  }
}

/**
 * Implementation of hook_form_submit()
 * Adding a new gift certificate
 */
function uc_gift_certificate_add_form_submit($form, &$form_state) {
  $use_auto = variable_get('uc_use_autocomplete_user_fields', true);
  $is_new_cert = !isset($form_state['values']['certificate_id']);
  $is_new_user = $form_state['values']['new_user_checkbox'];
  $do_mail = $form_state['values']['mail_user_checkbox'];

  //drupal_set_message($form['cert_code']);
  // Make sure we don't generate a new cert code if we are editing an existing certificate.
  if (!isset($form_state['values']['cert_code'])) {
    $cert_code = uc_gift_certificate_mKey();
  }
  else {
    $cert_code = $form_state['values']['cert_code'];
  }

  if ($is_new_cert) {
    $success = drupal_write_record('uc_gift_certificates', $form_state['values']);
    $certificate_id = db_last_insert_id('uc_gift_certificates', 'certificate_id');
    $cert_code = $certificate_id."-".$cert_code;
    db_query("UPDATE {uc_gift_certificates} SET cert_code = '%s' WHERE certificate_id = %d", $cert_code, $certificate_id);
  }
  else {
    $certificate_id = $form_state['values']['certificate_id'];
    $success = drupal_write_record('uc_gift_certificates', $form_state['values'], 'certificate_id');
  }

  if ($success) {
    if ($is_new_user) {
      $mail = $form_state['values']['new_user_email'];
    }
    else {
      $cert = uc_gift_certificate_load($certificate_id);
      $user = user_load(array('uid' => $cert->user_id));
      $mail = $user->mail;
    }

    uc_gift_certificate_issue_cert($mail, $certificate_id);
    if ($do_mail) {
      uc_gift_certificate_mail_cert($mail, $certificate_id, $form_state['values']['msg_to_recipient']);
    }

    if ($is_new_cert) {
      drupal_set_message('Successfully added the gift certificate.');
    }
    else {
      drupal_set_message('Successfully updated the certificate.');
    }
  }

  drupal_goto('admin/store/gift_certificates');
}

/**
 * Confirm certificate deletion
 */
function uc_gift_certificate_delete_confirm($form_state, $cid) {
 $form['cid'] = array('#type' => 'value', '#value' => $cid);

 return confirm_form($form,
  t('Confirm Deletion of Gift Certificate %cid', array('%cid' => $cid)),
    'admin/store/gift_certificates',
  t('Deleting this gift certificate is irreversible.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Delete gift certificate
 */
function uc_gift_certificate_delete_confirm_submit($form, &$form_state) {
  $cert_id = $form_state['values']['cid'];
  $query = "DELETE FROM {uc_gift_certificates} WHERE certificate_id = '" . $cert_id . "'";
  if (db_query($query)) {
    drupal_set_message("Gift Certificate Deleted Successfully");
  }
  else {
    drupal_set_message("Error deleting gift certificate");
  }
  $form_state['redirect'] = 'admin/store/gift_certificates';
}

/**
 * Hook_settings
 */
function uc_gift_certificate_settings() {
  $form['email'] = array(
    '#type' => 'fieldset',
    '#title' => t('Email settings'),
    '#description' => t('You can customize the content of the gift certificate email by copying uc_gift_certificate.tpl.php to your theme directory.'),
  );
  $form['email']['uc_gc_email_new_user_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Email subject for a new user'),
    '#default_value' => variable_get('uc_gc_email_new_user_subject', uc_gift_certificate_get_setting_defaults('uc_gc_email_new_user_subject')),
    '#description' => t('Email subject for a gift certificate to a new user'),
  );
  $form['email']['token_help'] = array(
    '#type' => 'fieldset',
    '#title' => t('Replacement patterns'),
    '#description' => t('You can make use of the replacement patterns in the subjects and the template file.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  foreach (array('global', 'gift_certificate', 'user') as $name) {
    $form['email']['token_help'][$name] = array(
      '#type' => 'fieldset',
      '#title' => t('@name replacement patterns', array('@name' => drupal_ucfirst($name))),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['email']['token_help'][$name]['content'] = array(
      '#value' => theme('token_help', $name),
    );
  }
  $form['email']['uc_gc_email_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Email subject for an existing user'),
    '#default_value' => variable_get('uc_gc_email_subject', uc_gift_certificate_get_setting_defaults('uc_gc_email_subject')),
    '#description' => t('Email subject for a gift certificate to an existing user'),
  );

  $result = db_query('SELECT * FROM {uc_attributes} ORDER BY ordering, name');
  $options = array();
  while ($attrib = db_fetch_object($result)) {
    $options[$attrib->aid] = $attrib->name;
  }
  $form['uc_gift_certificate_message_aid'] = array(
    '#type' => 'select',
    '#title' => t('"Message to Recipient" attribute'),
    '#default_value' => variable_get('uc_gift_certificate_message_aid', ''),
    '#options' => $options,
    '#description' => t('Choose which attribute you are using for the "Message to Recipient" value.'),
    '#required' => TRUE,
  );
  $form['uc_gift_certificate_email_aid'] = array(
    '#type' => 'select',
    '#title' => t('"Recipient\'s Email Address" attribute'),
    '#default_value' => variable_get('uc_gift_certificate_email_aid', ''),
    '#options' => $options,
    '#description' => t('Choose which attribute you are using for the "Recipient\'s Email Address" value.'),
    '#required' => TRUE,
  );

  $options = array();
  foreach (uc_order_status_list('general') as $status) {
    $options[$status['id']] = $status['title'];
  }
  foreach (uc_order_status_list('specific') as $status) {
    $options[$status['id']] = $status['title'];
  }
  $form['uc_gift_certificate_order_status_issue'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Order Status for Issuing'),
    '#default_value' => variable_get('uc_gift_certificate_order_status_issue', uc_gift_certificate_get_setting_defaults('uc_gift_certificate_order_status_issue')),
    '#options' => $options,
    '#description' => t('Choose what the order status must be in order the for the gift certificate to be issued (upon purchase).'),
  );
  $form['uc_gift_certificate_order_status_deduct'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Order Status for Deducting'),
    '#default_value' => variable_get('uc_gift_certificate_order_status_deduct', uc_gift_certificate_get_setting_defaults('uc_gift_certificate_order_status_deduct')),
    '#options' => $options,
    '#description' => t('Choose what the order status must be in order the for the gift certificate to be deducted (upon using it).'),
  );

  $form['uc_use_autocomplete_user_fields'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use Autocomplete Fields'),
    '#default_value' => variable_get('uc_use_autocomplete_user_fields', true),
    '#description' => t('Use autocomplete fields for choosing which users are associated with a given gift certificate.'),
  );
  return system_settings_form($form);
}

/**
 * Issue a new gift certificate
 */
function uc_gift_certificate_issue() {
  global $user;

  $num_rows = db_result(db_query("SELECT COUNT(*) FROM {uc_gift_certificates} WHERE purchaser_id = %d", $user->uid));
  $result = db_query("SELECT * FROM {uc_gift_certificates} WHERE purchaser_id = %d", $user->uid);
  $rows = array();

  while ($row = db_fetch_object($result)) {
    if ($num_rows == 1) {
      drupal_goto("gift_certificates/$row->certificate_id/issue");
    }
    $rows[] = array($row->name, $row->value, l(t('send'), "gift_certificates/$row->certificate_id/issue"));
  }
  $header = array(array('data' => 'Name', 'width' => '100'),  array('data' => 'Value', 'width' => '60'),  array('data' => '', 'width' => 50));
  $output .= theme('table',$header, $rows);

  if ($output == null) {
    $output = "<p>There are currently no gift certificates in the system.</p>";
  }
  return $output;
}
